// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package hpb_unittest.someotherpackage;

import "google/protobuf/compiler/hpb/tests/test_model.proto";

// Define extension that is extending proto outside this package with a type
// defined in different file.

extend TestModel {
  optional ThemeExtension styling = 13001;
}
